#ifndef __MD2_H
#define __MD2_H


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gl/gl.h>				
#include <gl/glu.h>				

#include "texture.h"
#include "vector.h"

#define MAX_FRAMES 512

enum modelState_t
{
	MODEL_IDLE,		
	MODEL_CROUCH,	
	MODEL_RUN,		
	MODEL_JUMP,		
	MODEL_FIRE		
};

typedef struct
{
   float point[3];
} vector_t;

vector_t operator-(vector_t a, vector_t b);
vector_t operator*(float f, vector_t b);
vector_t operator/(vector_t a, vector_t b);
vector_t operator+(vector_t a, vector_t b);

void CalculateNormal( float *p1, float *p2, float *p3 );

typedef struct
{
   float s;
   float t;
} texCoord_t;

typedef struct
{
   short s;
   short t;
} stIndex_t;

typedef struct
{
   unsigned char v[3];
   unsigned char normalIndex;	
} framePoint_t;

typedef struct
{
  float s;
  float t;
  int index;

} md2_glcmd_t;

typedef struct
{
   float scale[3];
   float translate[3];
   char name[16];
   framePoint_t fp[1];
} frame_t;

typedef struct
{
   unsigned short meshIndex[3];		
   unsigned short stIndex[3];		
} mesh_t;

typedef struct
{
   int ident;		 
   int version;	 
   int skinwidth;    
   int skinheight;   
   int framesize;    
   int numSkins;     
   int numXYZ;       
   int numST;        
   int numTris;      
   int numGLcmds;
   int numFrames;    
   int offsetSkins;  
   int offsetST;     
   int offsetTris;   
   int offsetFrames; 
   int offsetGLcmds; 
   int offsetEnd;    
} modelHeader_t;


class CMD2Model 
{
private:
     int numFrames;			
     int numVertices;         
     int numTriangles;        
     int numST;               
     int numGLcmd;
     int frameSize;            
     mesh_t *triIndex;         
     texCoord_t *st;           
     vector_t *vertexList;     
     int * glCmd;
     void SetupSkin(CTexture *thisTexture);

protected:
	modelState_t modelState;	
     int nextFrame;           

public:
            CTexture *modelTex;      
int currentFrame; 
     float interpol;           

     CMD2Model();         
     ~CMD2Model();        

      int Load(char *modelFile, char *skinFile);
      int LoadModel(char *modelFile);
      int LoadSkin(char *skinFile);
      int SetTexture(CTexture *texture);
      int AnimateModel(int startFrame, int endFrame, float percent);
      int RenderFrame(int keyFrame);
      void Unload();
 	  void SetState(modelState_t state);	
 	  modelState_t GetState();
};

#endif
